#include "corner.h"
#include "prototypes.h"
#include "tools.h"

Corner::Corner(Edge* new_e1, Edge* new_e2): e1(new_e1), e2(new_e2) {

}

void Corner::draw() {
	for(uint16_t led: this->_leds) {
		leds[led] = this->color;
	}
}

void Corner::_infect(std::vector<Corner*>* neighbours) {
	for(Corner* c : *neighbours) {
		if (c==nullptr) continue;
		c->blend_to(this->target_color, this->effect_id, effect_speed);
	}
}

void Corner::step() {
	if (this->color_blend_state < 255) {
		this->color = blend(this->source_color, this->target_color, color_blend_state);
		this->color_blend_state = (this->color_blend_state + effect_speed > 255) ? 255 : this->color_blend_state + effect_speed;
		if (this->color_blend_state==255) {
			this->color = this->target_color;
		}
	}
}

void Corner::infect(uint8_t infect_short_level, uint8_t infect_long_level) {
	if (reached_level(infect_short_level)) {
		_infect(&_short_neighbours);
	}
	if (reached_level(infect_long_level)) {
		_infect(&_long_neighbours);
	}
}

bool Corner::reached_level(uint8_t level) {
	uint8_t old_cbs = color_blend_state >= effect_speed ? color_blend_state - effect_speed : 0;
	return (old_cbs < level && color_blend_state >= level);
}

/*bool Corner::loop() {
	if (this->color_blend_state < 255) {
		this->color = blend(this->source_color, this->target_color, color_blend_state);
		if (255 - color_blend_state >= effect_speed) {
			color_blend_state += effect_speed;
		} else {
			color_blend_state = 255;
		}
		
		if (color_blend_state >= 200 && color_blend_state-effect_speed<200) {
			infect(&_long_neighbours);
		}

		if (color_blend_state>=75 && color_blend_state-effect_speed<75) {
			infect(&_short_neighbours);
		}

		if (color_blend_state==255) {
			this->color = this->target_color;
		}
	}

	this->draw();

	return color_blend_state < 255;
}*/

void Corner::blend_to(CRGB target, uint16_t eid, uint8_t effect_speed) {
	LOGln("blendTo called. Corner: %p, target: %d,%d,%d, eid: %d, 'old' effect_id: %d, speed: %d", this, target.r, target.g, target.b, eid, effect_id, effect_speed);
	if (eid==0) {
		this->effect_id = random16();
		LOGln("Set effect_id to %d", this->effect_id);
	} else {
		if (this->effect_id == eid) {
			LOGln("'Old' effect. Doing nothing.");
			return;
		}
		this->effect_id = eid;
	}

	if (effect_speed==0) {
		effect_speed = random8(3) + 1;
	}
	this->effect_speed = effect_speed;
	
	this->source_color = this->color;
	this->target_color = target;
	this->color_blend_state = 0;
}

void Corner::add_led(uint16_t led) {
	_leds.push_back(led);
}

void Corner::merge_leds(Corner* c) {
	for(uint16_t led : c->_leds) {
		_leds.push_back(led);
	}
}

bool Corner::is_finished() {
	return color_blend_state < 255;
}

void Corner::set_color(CRGB color) {
	this->color = color;
	this->color_blend_state = 255;
}